import traceback

from cpl_core.configuration import ConfigurationModelABC
from cpl_core.console import Console
from cpl_query.extension import List


class BotStartupSettings(ConfigurationModelABC):
    def __init__(self):
        ConfigurationModelABC.__init__(self)

        self._cache_max_messages = 1000
        self._technicians: List[int] = List(int)

    @property
    def cache_max_messages(self) -> int:
        return self._cache_max_messages

    @property
    def technicians(self) -> List[int]:
        return self._technicians

    def from_dict(self, settings: dict):
        try:
            self._cache_max_messages = int(settings["CacheMaxMessages"])
            self._technicians = settings["Technicians"]
        except Exception as e:
            Console.error(f"[ ERROR ] [ {__name__} ]: Reading error in {type(self).__name__} settings")
            Console.error(f"[ EXCEPTION ] [ {__name__} ]: {e} -> {traceback.format_exc()}")
