from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC

from bot_core.configuration.bot_settings import BotSettings
from bot_core.helper.settings_helper import SettingsHelper
from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.data_seeder_abc import DataSeederABC
from bot_data.service.db_config import DBConfigService
from bot_data.service.seeder_service import SeederService
from modules.base.abc.base_helper_abc import BaseHelperABC
from modules.base.configuration.base_server_settings import BaseServerSettings
from modules.base.configuration.base_settings import BaseSettings
from modules.boot_log.configuration.boot_log_settings import BootLogSettings
from modules.level.configuration.level_settings import LevelSettings
from modules.permission.configuration.permission_settings import PermissionSettings


class ConfigSeeder(DataSeederABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: DatabaseLogger,
        seeder: SeederService,
        bot: DiscordBotServiceABC,
        db_context: DatabaseContextABC,
        db_config: DBConfigService,
        base_helper: BaseHelperABC,
    ):
        DataSeederABC.__init__(self)
        self._config = config

        self._logger = logger
        self._seeder = seeder
        self._bot = bot
        self._db_context = db_context
        self._db_config = db_config
        self._base_helper = base_helper

    async def seed(self):
        self._logger.debug(__name__, f"Load config from database")
        try:
            cfg = self._db_config.get_st_bot()

            bot_settings = BotSettings()
            bot_settings.from_dict(cfg)

            base_settings = BaseSettings()
            base_settings.from_dict(cfg)

            boot_log_settings = BootLogSettings()
            boot_log_settings.from_dict(cfg)

            level_settings = LevelSettings()
            level_settings.from_dict(cfg)

            permission_settings = PermissionSettings()
            permission_settings.from_dict(cfg)

            SettingsHelper.configure_settings_with_sub_settings(
                self._config, BotSettings, lambda x: x.servers, lambda x: x.id
            )
            SettingsHelper.configure_settings_with_sub_settings(
                self._config, BaseSettings, lambda x: x.servers, lambda x: x.id
            )
            SettingsHelper.configure_settings_with_sub_settings(
                self._config, BootLogSettings, lambda x: x.servers, lambda x: x.id
            )
            SettingsHelper.configure_settings_with_sub_settings(
                self._config, LevelSettings, lambda x: x.servers, lambda x: x.id
            )
            SettingsHelper.configure_settings_with_sub_settings(
                self._config, PermissionSettings, lambda x: x.servers, lambda x: x.id
            )
        except Exception as e:
            self._logger.fatal(__name__, "Cannot load config from db", e)

        try:
            for guild in self._bot.guilds:
                settings: BaseServerSettings = self._base_helper.get_config(guild.id)
                if settings is None:
                    self._db_config.create_config_for_guild(guild.id)
        except Exception as e:
            self._logger.fatal(__name__, "Cannot load or create guild config from db", e)

        self._logger.info(__name__, f"Loaded config from database")
