from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class ConfigMigration(MigrationABC):
    name = "1.0_ConfigMigration"

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")

        self._cursor.execute(
            f"""
            CREATE TABLE IF NOT EXISTS `ST_Bot` (
                `Id` BIGINT NOT NULL AUTO_INCREMENT,
                `WaitForRestart` BIGINT NOT NULL DEFAULT 8,
                `WaitForShutdown` BIGINT NOT NULL DEFAULT 8,
                PRIMARY KEY(`Id`)
            );
        """
        )

        self._cursor.execute(
            f""" 
            CREATE TABLE IF NOT EXISTS `ST_Servers` (
                `Id` BIGINT NOT NULL AUTO_INCREMENT,
                `ServerId` BIGINT NOT NULL,
                `MessageDeleteTimer` BIGINT NOT NULL DEFAULT 4,
                -- base
                `MaxVoiceStateHours` BIGINT NOT NULL DEFAULT 6,
                `XpPerMessage` BIGINT NOT NULL DEFAULT 1,
                `XpPerReaction` BIGINT NOT NULL DEFAULT 1,
                `MaxMessageXpPerHour` BIGINT NOT NULL DEFAULT 40,
                `XpPerOntimeHour` BIGINT NOT NULL DEFAULT 20,
                `XpPerEventParticipation` BIGINT NOT NULL DEFAULT 20,
                `AFKCommandChannelId` BIGINT NOT NULL DEFAULT 1,
                `HelpCommandReferenceUrl` VARCHAR(512) NOT NULL DEFAULT 'https://git.sh-edraft.de/sh-edraft.de/kd_discord_bot/wiki/Befehle',
                `HelpVoiceChannelId` BIGINT NOT NULL DEFAULT 1,
                -- boot log
                `LoginMessageChannelId` BIGINT NOT NULL,
                -- level
                `ChangedLevelNotificationChannelId` BIGINT NOT NULL,
                PRIMARY KEY(`Id`),
                FOREIGN KEY (`ServerId`) REFERENCES `Servers`(`ServerId`)
            );
        """
        )

        self._cursor.execute(
            f""" 
            CREATE TABLE IF NOT EXISTS `ST_Servers_AFKChannelIds` (
                `Id` BIGINT NOT NULL AUTO_INCREMENT,
                `ST_Server` BIGINT NOT NULL,
                `ChannelId` BIGINT NOT NULL,
                PRIMARY KEY(`Id`),
                FOREIGN KEY (`ST_Server`) REFERENCES `ST_Servers`(`Id`)
            );
        """
        )

        self._cursor.execute(
            f""" 
            CREATE TABLE IF NOT EXISTS `ST_Servers_PingURLs` (
                `Id` BIGINT NOT NULL AUTO_INCREMENT,
                `ST_Server` BIGINT NOT NULL,
                `URL` VARCHAR(512) NOT NULL,
                PRIMARY KEY(`Id`),
                FOREIGN KEY (`ST_Server`) REFERENCES `ST_Servers`(`Id`)
            );
        """
        )

        self._cursor.execute(
            f""" 
            CREATE TABLE IF NOT EXISTS `ST_Servers_Permissions` (
                `Id` BIGINT NOT NULL AUTO_INCREMENT,
                `ST_Server` BIGINT NOT NULL,
                `RoleId` BIGINT NOT NULL,
                `RoleType` BIGINT NOT NULL DEFAULT 1,
                PRIMARY KEY(`Id`),
                FOREIGN KEY (`ST_Server`) REFERENCES `ST_Servers`(`Id`)
            );
        """
        )

        self._cursor.execute(
            f""" 
            CREATE TABLE IF NOT EXISTS `ST_RoleTypes` (
                `Id` BIGINT NOT NULL AUTO_INCREMENT,
                `Name` VARCHAR(10) NOT NULL,
                PRIMARY KEY(`Id`)
            );
        """
        )

        self._cursor.execute(
            f""" 
            INSERT INTO `ST_RoleTypes` 
            (`Id`, `Name`) 
            VALUES 
            (1, 'Moderator'),
            (2, 'Admin');
        """
        )

    def downgrade(self):
        self._logger.debug(__name__, "Running downgrade")
        self._cursor.execute("DROP TABLE `ST_Bot`;")
        self._cursor.execute("DROP TABLE `ST_Servers`;")
        self._cursor.execute("DROP TABLE `ST_Servers_AFKChannelIds`;")
        self._cursor.execute("DROP TABLE `ST_Servers_PingURLs`;")
        self._cursor.execute("DROP TABLE `ST_Servers_Permissions`;")
        self._cursor.execute("DROP TABLE `ST_RoleTypes`;")
